package com.winhc.utils;

import cn.hutool.core.lang.Tuple;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.google.common.collect.ImmutableMap;
import com.winhc.common.enums.CompanyEnum;
import com.winhc.db.mongodb.dataobject.NodeRelationError;
import com.winhc.pojo.MergePerson;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.neo4j.driver.Record;
import org.neo4j.driver.Result;
import org.neo4j.driver.Session;

import java.util.*;
import java.util.stream.Collectors;

import static com.winhc.utils.DateUtil.formatDate_YYYY_MM_DD_HH_MM_SS;

/**
 * @author π
 * @Description:
 * @date 2021/1/11 21:31
 */
public class CompanyUtils {
    public static List<Map<String, Object>> map(List<ConsumerRecord<?, ?>> records) {
        return records.stream().filter(r -> (r != null && r.value() != null)).map(r -> {
            Map<String, Object> m = JSONUtil.parseObj(r.value().toString());
            return m;
        }).collect(Collectors.toList());
    }

    public static List<Map<String, Object>> mapPlus(List<ConsumerRecord<?, ?>> records) {
        return records.stream().filter(r -> (r != null && r.value() != null)).map(r -> {
            JSONObject jsonObject = JSONObject.parseObject(r.value().toString());
            String person_name = jsonObject.getString("person_name");
            JSONArray company_infos = jsonObject.getJSONArray("company_infos");
            ArrayList<String> list = new ArrayList<>();
            HashMap<String, Object> map = new HashMap<>();
            for (Object company_info : company_infos) {
                JSONObject jsonObject1 = (JSONObject) company_info;
                String company_id = jsonObject1.getString("company_id");
                list.add(company_id);
            }
            map.put("name", person_name);
            map.put("company_ids", listToString(list));
            return map;
        }).collect(Collectors.toList());
    }

    public static List<Map<String, Object>> mapPlusV1(List<ConsumerRecord<?, ?>> records) {
        return records.stream().filter(r -> (r != null && r.value() != null)).map(r -> {
            HashMap<String, Object> map = new HashMap<>();
            JSONObject jsonObject = JSONObject.parseObject(r.value().toString());
            String merge_human_pid = jsonObject.getString("merge_human_pid");
            String deleted_human_pid = jsonObject.getString("deleted_human_pid");

            map.put("merge_human_pid", merge_human_pid);
            map.put("deleted_human_pid", deleted_human_pid);
            return map;
        }).collect(Collectors.toList());
    }

    public static String listToString(List<String> list) {
        if (list.size() == 0) return "";
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < list.size(); i++) {
            sb.append(list.get(i));
            if (i != list.size() - 1) {
                sb.append("&");
            }
        }
        return sb.toString();
    }

    public static List<Map<String, Object>> filterList(List<Map<String, Object>> list, String type) {
        return list.stream().filter(r -> (r.getOrDefault("topic_type", "-1").equals(type))).collect(Collectors.toList());
    }

    public static List<String> getMergeIds(List<Map<String, Object>> list) {
        return list.stream()
                .filter(r -> r.getOrDefault("start_id", "0").toString().length() == 33)
                .collect(Collectors.toMap(t -> t.getOrDefault("start_id", "0").toString(), t -> t, (n, o) -> n))
                .values().stream().map(x -> {
                    Map<String, String> m = (Map) x;
                    ImmutableMap<String, String> m2 = ImmutableMap.of(
                            "person_id", m.get("start_id")
                            , "name", m.get("start_name")
                            , "topic_type", "800"
                    );
                    return JSONObject.toJSONString(m2, SerializerFeature.WriteMapNullValue);
                }).collect(Collectors.toList());
    }

    public static List<Map<String, Object>> getMergeIds2(List<Map<String, Object>> list) {
        return list.stream()
                .filter(r -> r.getOrDefault("start_id", "0").toString().length() == 33)
                .collect(Collectors.toMap(t -> t.getOrDefault("start_id", "0").toString(), t -> t, (n, o) -> n))
                .values().stream().map(x -> {
                    Map<String, Object> m = x;
                    String person_id = m.get("start_id").toString();
                    String person_name = m.get("start_name").toString();
                    return ImmutableMap.<String, Object>of(
                            "_id", person_id,
                            "update_time", formatDate_YYYY_MM_DD_HH_MM_SS(),
                            "data", ImmutableMap.of(
                                    "person_id", person_id
                                    , "name", person_name
                                    , "topic_type", "800"
                            )
                    );
                }).collect(Collectors.toList());
    }

    public static List<NodeRelationError> toMessage(List<ConsumerRecord> records, String errorMessage) {
        return records.stream().filter(r -> (r != null && r.value() != null)).map(r -> {
            String consumerMessage = r.value().toString();
            Map<String, Object> m = JSONUtil.parseObj(consumerMessage);
            Object topic_type = m.getOrDefault("topic_type", "-1");
            Date date = new Date();
            NodeRelationError n1 = new NodeRelationError();
            n1.setCreateTime(date);
            n1.setUpdateTime(date);
            n1.setConsumerMessage(consumerMessage);
            n1.setErrorMessage(errorMessage);
            n1.setStatus(0);
            n1.setTopicType(String.valueOf(topic_type));
            return n1;
        }).collect(Collectors.toList());
    }

    public static List<String> toMessage(List<ConsumerRecord> records) {
        return records.stream()
                .filter(r -> (r != null && r.value() != null))
                .map(r -> r.value().toString())
                .collect(Collectors.toList());
    }

    public static String writeNeo4j(Session session, String cql, HashMap<String, Object> parameters) {
        String data = session.writeTransaction(tx -> {
            Result result = tx.run(cql, parameters);
            return "success";
        });
        return data;
    }

    public static String writeNeo4j2(Session session, String cql, HashMap<String, Object> parameters) {
        List<Record> dataList = session.writeTransaction(tx -> {
            Result result = tx.run(cql, parameters);
            return result.list();
        });
        Map<String, MergePerson> data = dataList.stream()
                .map(record -> JSON.parseObject(JSON.toJSONString(record.asMap()), MergePerson.class))
                .collect(Collectors.toMap(MergePerson::getId, t -> t, (n, o) -> n));
        Collection<MergePerson> values = data.values();
        return values.size() > 0 ? JSONObject.toJSONString(data.values()) : null;
    }

    public static String getIncrPersonLabel() {
        return CompanyEnum.Lable.新增.code + DateUtil.getDateBefore(-1).replace("-", "");
    }

    public static String getIncrPersonLabel(String label) {
        return label + DateUtil.getDateBefore(-1).replace("-", "");
    }

    public static String getIncrPersonLabelV2(String label) {
        return getIncrPersonLabelV2(label, 10);
    }

    public static String getIncrPersonLabelV2(String label, int split_hour) {
        GregorianCalendar calendar = new GregorianCalendar();
        int now_hour = calendar.get(Calendar.HOUR_OF_DAY);
        int gap = now_hour > split_hour ? 0 : -1;
        return label + DateUtil.getDateBefore(gap).replace("-", "");
    }

    public static Boolean isWindows() {
        return System.getProperty("os.name").contains("Windows");
    }


    public static void main(String[] args) {
        System.out.println(getIncrPersonLabelV2("新增", 10));
        System.out.println(getIncrPersonLabelV2("新增"));
    }
}
